var EchartsCRMV3Theme = {
    color: [
        '#2ec7c9', '#b6a2de', '#5ab1ef', '#ffb980', '#d87a80',
        '#8d98b3', '#e5cf0d', '#97b552', '#95706d', '#dc69aa',
        '#07a2a4', '#9a7fd1', '#588dd5', '#f5994e', '#c05050',
        '#59678c', '#c9ab00', '#7eb00a', '#6f5553', '#c14089'
    ],
    loadingText: 'Loading...', noDataText: 'No data',
    tooltip: {trigger: 'item', position: calcEchartsTooltipPosition, backgroundColor: 'rgba(255,255,255,0.9)', borderWidth: 2, textStyle: {color: "#333"}, extraCssText: 'z-index:1031;'}
};

if (AppConfig["templateThemeVersion"] >= 2) {
    EchartsCRMV3Theme["color"] = [
        '#759aa0', '#e69d87', '#8dc1a9', '#ea7e53', '#eedd78',
        '#73a373', '#73b9bc', '#7289ab', '#91ca8c', '#f49f42',
        '#dd6b66'
    ];
}

var EchartsCRMV3 = {

};

var EchartsMapsDataCRMV3 = {

};

var EchartsMapsDataCRMV3Registering = {

};

function renderChart(container, options) {
    var chart = echarts.init(document.getElementById(container), EchartsCRMV3Theme);
    chart.setOption(options, true);
    chart.on("click", function (param) {
        if (typeof window["anyEchartsClick"] === "function") {
            return window["anyEchartsClick"](param, chart, container);
        }
        var functionName = container + "Click";
        if (typeof window[functionName] === "function") {
            return window[functionName](param, chart, container);
        } else {
            return false;
        }
    });
    chart.on("legendselectchanged", function (param) {
        if (typeof window["anyEchartsLegendClick"] === "function") {
            return window["anyEchartsLegendClick"](param, chart, container);
        } else {
            return false;
        }
    });
    EchartsCRMV3[container] = chart;
}

$(window).resize(function () {
    resizeEcharts();
});

function resizeEcharts() {
    $.each(EchartsCRMV3, function (k, v) {
        v.resize();
    });
}

function restoreEcharts() {
    $.each(EchartsCRMV3, function (k, v) {
        v.dispatchAction({type: 'restore'});
    });
}

function renderPie(container, data) {
    var legendData = [];
    var emptyValues = 0;
    var formatter = pieTooltipFormater;
    if (typeof data.formatter !== "undefined") {
        formatter = (window[data.formatter] ? window[data.formatter] : data.formatter);
    }
    $(data.data).each(function (k, v) {
        if (v.value !== 0) {
            if (!data["hideLegend"]) {
                legendData.push(generateLegendItemObject(v.name, 'circle'));
            }
        } else {
            delete data.data[k]; /*No le gusta que le pasemos items con valor 0*/
            emptyValues = emptyValues + 1;
        }
    });

    if (emptyValues === data.data.length) {
        data.data = [];
    }

    var options = {
        title: generateTitleObject(data),
        tooltip: {formatter: formatter},
        legend: (data.showLeyend === false ? false : generateLegendObject(legendData, pieLegendFormatter)),
        toolbox: generateToolboxObject(data),
        calculable: false,
        series: [
            {
                center: [(typeof data.centerLeft !== "undefined" ? data.centerLeft : "50%"), (typeof data.centerTop !== "undefined" ? data.centerTop : "33%")],
                radius: (typeof data.radius !== "undefined" ? data.radius : "50%"),
                name: '', type: 'pie', data: data.data,
                itemStyle: {
                    normal: {label: {show: false}, labelLine: {show: false}},
                    emphasis: {label: {show: false}, labelLine: {show: false}}
                }
            }
        ]
    };
    if (typeof data.color !== "undefined") {
        options["color"] = data.color;
    }
    renderChart(container, options);
}

function renderBar(container, data) {
    var legendData = [];
    var series = [];
    var multiYAxis = 0;
    $(data.data).each(function (k, v) {
        legendData.push(generateLegendItemObject(v.name));
        series.push({name: v.name, type: (v.type ? v.type : 'bar'), areaStyle: (v.areaStyle ? v.areaStyle : null), yAxisIndex: (v.yAxisIndex ? v.yAxisIndex : 0), data: v.data, stack: v.stack, color: v.color, emphasis: {focus: (data["emphasisFocus"] ? data["emphasisFocus"] : "none")}});
        if (v.yAxisIndex && v.yAxisIndex > multiYAxis) {
            multiYAxis = v.yAxisIndex;
        }
    });
    var formatter = barTooltipFormater;
    if (typeof data.formatter !== "undefined") {
        formatter = (window[data.formatter] ? window[data.formatter] : data.formatter);
    }
    var xAxis = null;
    var yAxis = null;
    var showxAxis = (data.showxAxis === false ? false : true);
    if (data.orientation !== "horizontal")
    {
        xAxis = [{type: 'category', data: data.categories, show: showxAxis, boundaryGap: ((typeof data.boundaryGap !== "undefined") ? data.boundaryGap : true)}];
        yAxis = [{type: 'value', axisLabel: {formatter: axisLabelFormatter}}];
        for (var i = 1; i <= multiYAxis; i++) {
            yAxis.push({type: 'value', axisLabel: {formatter: axisLabelFormatter}});
        }
    } else {
        xAxis = [{show: showxAxis, type: 'value', axisLabel: {formatter: axisLabelFormatter}}];
        yAxis = [{type: 'category', data: data.categories}];
    }

    var options = {
        title: generateTitleObject(data),
        tooltip: {
            trigger: 'axis', formatter: formatter,
            axisPointer: {
                type: (data.categories.length > 1 ? 'cross' : 'shadow'),
                label: {formatter: function (params) {
                        return (isNaN(params["value"]) ? params["value"] : appNumberFormat(params["value"]));
                    }}
            }
        },
        legend: generateLegendObject(legendData),
        toolbox: generateToolboxObject(data),
        calculable: false,
        xAxis: xAxis,
        yAxis: yAxis,
        series: series
    };

    if (multiYAxis === 0) {
        options["grid"] = {
            x: (data["gridX"] ? data["gridX"] : "20%"), x2: (data["gridX2"] ? data["gridX2"] : "0%"),
            y: (data["gridY"] ? data["gridY"] : "5%"), y2: (data["gridY2"] ? data["gridY2"] : "0%"),
            width: (data["gridWidth"] ? data["gridWidth"] : "80%"), height: (data["gridHeight"] ? data["gridHeight"] : "75%")
        };
    } else {
        options["grid"] = {
            x: (data["gridX"] ? data["gridX"] : "15%"),
            y: (data["gridY"] ? data["gridY"] : "5%"), y2: (data["gridY2"] ? data["gridY2"] : "0%"),
            width: (data["gridWidth"] ? data["gridWidth"] : "80%"), height: (data["gridHeight"] ? data["gridHeight"] : "75%")
        };
    }

    if (typeof data.color !== "undefined") {
        options["color"] = data.color;
    }
    renderChart(container, options);
}

function renderSunburst(container, data) {
    var options = {
        title: generateTitleObject(data),
        toolbox: generateToolboxObject(data),
        tooltip: {formatter: sunburstTooltipFormater},
        series: {
            type: 'sunburst', sort: (data["sort"] ? data["sort"] : null),
            data: data["data"],
            center: [(typeof data.centerLeft !== "undefined" ? data.centerLeft : "50%"), (typeof data.centerTop !== "undefined" ? data.centerTop : "50%")],
            emphasis: {focus: 'ancestor'},
            levels: [{r0: '0%', r: '20%'}, {r0: '20%', r: (typeof data.radius1 !== "undefined" ? data.radius1 : "70%")}, {r0: (typeof data.radius1 !== "undefined" ? data.radius1 : "70%"), r: (typeof data.radius2 !== "undefined" ? data.radius2 : "100%")}],
            label: {show: (data["showLabel"] ? true : false), rotate: (data["labelRotate"] ? data["labelRotate"] : 'tangential'), position: "bottom", formatter: sunburstLabelFormater}
        }
    };
    renderChart(container, options);
}

function renderGauge(container, data) {

    var options = {
        title: generateTitleObject(data),
        tooltip: {show: false},
        toolbox: generateToolboxObject(data),
        series: [{
                type: 'gauge', min: data.min, max: data.max,
                startAngle: 180, endAngle: 0, splitNumber: 5,
                center: [(typeof data.centerLeft !== "undefined" ? data.centerLef : "50%"), (typeof data.centerTop !== "undefined" ? data.centerTop : "90%")],
                radius: 110,
                axisLine: {lineStyle: {color: data.ranges, width: 13}},
                axisTick: {
                    distance: 0, splitNumber: 2, length: 4,
                    lineStyle: {color: 'auto', width: 2}
                },
                axisLabel: {distance: 2, color: 'auto'},
                splitLine: {show: false},
                pointer: {
                    icon: 'path://M2090.36389,615.30999 L2090.36389,615.30999 C2091.48372,615.30999 2092.40383,616.194028 2092.44859,617.312956 L2096.90698,728.755929 C2097.05155,732.369577 2094.2393,735.416212 2090.62566,735.56078 C2090.53845,735.564269 2090.45117,735.566014 2090.36389,735.566014 L2090.36389,735.566014 C2086.74736,735.566014 2083.81557,732.63423 2083.81557,729.017692 C2083.81557,728.930412 2083.81732,728.84314 2083.82081,728.755929 L2088.2792,617.312956 C2088.32396,616.194028 2089.24407,615.30999 2090.36389,615.30999 Z',
                    length: '80%', width: 8,
                    itemStyle: {color: '#4e4e4e', opacity: 0.4}
                },
                title: {
                    show: true, offsetCenter: [0, '-40%'],
                    textStyle: {fontWeight: 'bolder'}
                },
                detail: {
                    formatter: gaugeTooltipFormater,
                    offsetCenter: ['0', '-25%'],
                    textStyle: {color: 'auto', fontSize: 30, fontWeight: 'normal'}
                },
                data: [{value: data.value, name: ''}]
            }
        ]
    };
    renderChart(container, options);
}

function renderPercentageChart(container, data) {

    var value = data["value"];
    var max = 100;
    if (data["max"] || data["max"] === 0) {
        max = data["max"];
    }
    value = parseFloat(value);
    max = parseFloat(max);

    var perc = 100;
    if (max === 0 && value === 0) {
        perc = 0;
    } else if (max !== 0) {
        perc = Math.round(parseFloat((value / max) * 100));
    }

    var perc100 = perc;
    if (perc > 100) {
        perc100 = 100;
    }

    if (data["color"]) {
        var color = data["color"];
    } else {
        var color = "#73a373";
        if (perc < 60) {
            color = "#CC5E5E";
        } else if (perc < 80) {
            color = "#ea7e53";
        }
    }

    var fontSize = (data["fontSize"] ? data["fontSize"] : 14);
    var percTitle = perc + '%';
    if (percTitle.length > 4) {
        fontSize = (fontSize - 3);
    }

    var options = {
        legend: {show: false},
        title: generateTitleObject(data),
        toolbox: {show: false},
        series: [{
                type: 'pie',
                center: [(typeof data["centerLeft"] !== "undefined" ? data["centerLeft"] : "50%"), (typeof data["centerTop"] !== "undefined" ? data["centerTop"] : "50%")],
                radius: (data["radius"] ? data["radius"] : [20, 25]),
                label: {show: false, position: 'center', padding: [15, 0, 0, 0], fontSize: fontSize, color: color},
                data: [
                    {name: percTitle, value: perc100, itemStyle: {normal: {color: (data["onlyPercentage"] ? "transparent" : color)}}, label: {show: true}},
                    {name: '', value: (100 - perc100), itemStyle: {normal: {color: (data["onlyPercentage"] ? "transparent" : '#ccc')}}}
                ]
            }]
    };
    renderChart(container, options);
}

function renderFunnel(container, data) {
    var legendData = [];
    var max = 0;
    $(data.data).each(function (k, v) {
        if (max < v.value)
            max = v.value;
        legendData.push(generateLegendItemObject(v.name, 'circle'));
    });
    var options = {
        title: generateTitleObject(data),
        tooltip: {formatter: funnelTooltipFormater},
        toolbox: generateToolboxObject(data),
        legend: generateLegendObject(legendData),
        calculable: false,
        series: [
            {
                type: 'funnel', sort: 'none',
                x: 0, x2: 0,
                y: 25, y2: 35,
                min: 0, max: max,
                itemStyle: {normal: {label: {show: false}, labelLine: {show: false}}, emphasis: {label: {show: false}, labelLine: {show: false}}},
                data: data.data
            }
        ]
    };
    renderChart(container, options);
}

function renderLine(container, data) {
    var legendData = [];
    var series = [];
    var formatter = lineTooltipFormater;
    if (typeof data.formatter !== "undefined") {
        formatter = (window[data.formatter] ? window[data.formatter] : data.formatter);
    }

    $(data.data).each(function (k, v) {
        var inLegend = false;
        $(v.data).each(function (k, v) {
            if (v !== 0) {
                inLegend = true;
            }
        });
        if (inLegend) {
            legendData.push(generateLegendItemObject(v.name));
        }
        series.push({
            name: v.name,
            stack: (typeof data.stacked !== "undefined" ? "1" : null),
            type: 'line',
            areaStyle: (typeof data.areaStyle !== "undefined" ? data.areaStyle : null),
            data: v.data,
            color: v.color,
            smooth: ((typeof data.smooth !== "undefined") ? data.smooth : false)
        });
    });

    var options = {
        grid: {
            x: (data.gridX ? data.gridX : "8%"),
            x2: (data.gridX2 ? data.gridX2 : "0%"),
            y: (data.gridY ? data.gridY : "5%"),
            y2: (data.gridY2 ? data.gridY2 : "0%"),
            width: (data.gridWidth ? data.gridWidth : "92%"),
            height: (data.gridHeight ? data.gridHeight : "65%")
        },
        title: generateTitleObject(data),
        tooltip: {
            trigger: 'axis', formatter: formatter,
            axisPointer: {
                type: 'cross',
                label: {formatter: function (params) {
                        return (isNaN(params["value"]) ? params["value"] : appNumberFormat(params["value"]));
                    }
                }
            }
        },
        legend: generateLegendObject(legendData, lineLegendFormatter),
        toolbox: generateToolboxObject(data),
        calculable: false,
        xAxis: [
            {
                show: (data.showxAxis === false ? false : true),
                type: 'category',
                boundaryGap: (typeof data.boundaryGap !== "undefined" ? data.boundaryGap : true),
                data: data.categories
            }
        ],
        yAxis: [
            {
                type: 'value',
                axisLabel: {
                    formatter: axisLabelFormatter
                }
            }
        ],
        series: series
    };
    if (typeof data.color !== "undefined") {
        options["color"] = data.color;
    }
    renderChart(container, options);
}

function renderRadar(container, data) {

    var legendData = [];
    $(data.data).each(function (k, v) {
        if (v.value !== 0) {
            legendData.push(generateLegendItemObject(v.name, 'circle'));
        }
    });

    var options = {
        title: generateTitleObject(data),
        tooltip: {formatter: (typeof data.formatter !== "undefined" ? window[data.formatter] : function (params, ticket, callback) {
                var radarIndicator = EchartsCRMV3[container].getOption("data")["radar"][0]["indicator"];
                var total = 0;
                var res = "<table>";
                res += '<tr><td colspan="2"><span style="background-color:' + params.color + ';">&nbsp;&nbsp;</span> ' + params["data"]["name"] + '</td></tr>';
                $.each(params["data"]["value"], function (k, v) {
                    if (v !== 0) {
                        res += '<tr>';
                        res += '<td>' + radarIndicator[k]["name"] + ':</td>';
                        res += '<td align="right">' + numberFormat(v) + (data["percentageRadar"] ? "%" : "") + '</td>';
                        res += '</tr>';
                        total += v;
                    }
                });
                res += '<tr><td><b>' + (data["percentageRadar"] ? "Av." : "Total") + '</b>:</td><td align="right"><b>' + (data["percentageRadar"] ? numberFormat((total / params["data"]["value"].length)) + "%" : numberFormat(total)) + '</b></td></tr>';
                res += "</table>";
                return res;
            })},
        legend: (data.showLeyend === false ? false : generateLegendObject(legendData, pieLegendFormatter)),
        radar: {name: {textStyle: {color: '#fff', backgroundColor: '#999', borderRadius: 3, padding: [3, 5]}}, indicator: data["indicator"], radius: (data["radius"] ? data["radius"] : "65%")},
        series: [{type: 'radar', areaStyle: {}, data: data["data"]}]
    };

    renderChart(container, options);
}

function renderTreeMap(container, data) {

    var levels = [];
    if (data["showUpperLabel"]) {
        levels = [
            {
                itemStyle: {borderWidth: 0, gapWidth: 0},
                upperLabel: {show: false}
            },
            {
                itemStyle: {borderColor: '#ffffff', borderWidth: 2, gapWidth: 1},
                emphasis: {itemStyle: {borderColor: '#dedede'}},
                upperLabel: {show: true}
            },
            {
                itemStyle: {borderWidth: 0, gapWidth: 5}
            }
        ];
    }

    var options = {
        title: generateTitleObject(data),
        tooltip: {formatter: function (params, ticket, callback) {
                var res = '<table>';
                $.each(params["treePathInfo"], function (k, v) {
                    res += '<tr>';
                    var name = "";
                    if (k === 0) {
                        name = "<b>Total</b>";
                    } else {
                        if (k === 1) {
                            name = "<span style='background-color:" + params.color + ";'>&nbsp;&nbsp;</span> " + v["name"];
                        } else {
                            name = "&bull;&nbsp;</span> " + v["name"];
                        }
                        var dataIndexData = getEchartsDataIndexData(container, params["seriesIndex"], v["dataIndex"]);
                        if (dataIndexData !== false && dataIndexData["chartName"]) {
                            name = dataIndexData["chartName"];
                        }
                    }
                    res += '<td>' + name + "</td>";
                    res += '<td align="right">';
                    res += appNumberFormat(v["value"]);
                    if (params["data"]["currencyValue"] === 1) {
                        res += ' ' + AppConfig.currencySymbol;
                    }
                    res += '</td>';
                    res += '<td align="right">';
                    if (k !== 0) {
                        res += ' (' + appNumberFormat(100 * (v["value"] / params["treePathInfo"][k - 1]["value"])) + '%)';
                    }
                    res += '</td>';
                    res += '</tr>';
                });
                res += '</table>';
                return res;
            }
        },
        toolbox: generateToolboxObject(data),
        series: [{
                type: 'treemap', sort: 'desc', roam: (data["roam"] ? data["roam"] : 'move'),
                data: data["data"],
                label: {
                    show: (data["hideLabel"] ? false : true),
                    formatter: function (params) {
                        if (params["data"]["chartName"]) {
                            return params["data"]["chartName"];
                        }
                        return params["name"];
                    }
                },
                levels: levels
            }]
    };
    renderChart(container, options);
}

function renderEchartsMap(container, data) {

    $("#" + container).html('<div class="width-100 center padding" style="margin-top:50px;"><img src="/images/loading.gif"></div>');
    registerEchartsMap(data["country"], function (map) {
        var seriesData = [];
        $.each(data["data"], function (k, v) {
            seriesData.push({name: map["mapData"]["featuresById"][v["name"]], value: v["value"]});
        });
        var rangeColor = ['lightskyblue', 'yellow', 'orangered'];
        if (typeof data.rangeColor !== "undefined") {
            rangeColor = data.rangeColor;
        }

        var options = {
            title: generateTitleObject(data),
            tooltip: {formatter: echartsMapTooltipFormater},
            toolbox: generateToolboxObject(data),
            visualMap: {
                left: 'right', min: data["visualMapMin"], max: data["visualMapMax"], text: ['', ''], calculable: true,
                inRange: {color: rangeColor}
            },
            series: [
                {
                    name: data["seriesName"], type: 'map', roam: (data["roam"] ? data["roam"] : true), map: map["mapName"],
                    itemStyle: {areaColor: "#ffffff"},
                    emphasis: {label: {show: false}, itemStyle: {areaColor: "#ebebeb"}},
                    select: {label: {show: true}, itemStyle: {areaColor: "#eef2f4"}},
                    data: seriesData
                }
            ]
        };
        renderChart(container, options);
    });
    return true;
}

function renderEchartsDashboard() {
    if (typeof echartsDashboardData === "undefined") {
        return false;
    }
    var dashboard = echartsDashboardData;
    var navContainer = $("#" + dashboard["navContainer"]);
    var navId = dashboard["container"] + "_nav";
    navContainer.html("").append('<select id="' + navId + '" onChange="loadEchartsDashboardTab($(this));" class="' + navId + 'Selector width-100"></select>');
    var nav = $("#" + navId);
    var dashboardContainer = $("#" + dashboard["container"]);
    var contentId = dashboard["container"] + "_content";
    dashboardContainer.html("").append('<div id="' + contentId + '"></div>');
    var content = $("#" + contentId);
    $.each(dashboard["tabs"], function (kTab, tab) {
        var tabPaneId = dashboard["container"] + "___" + kTab;
        nav.append('<option value="' + tabPaneId + '">' + tab["title"] + '</option>');
        content.append('<div id="' + tabPaneId + '" class="' + contentId + ' row"></div>');
    });

    if (!nav.find("option").length || nav.find("option").length <= 1) {
        nav.hide();
    }

    if (getLocalStorageItem(dashboard["type"] + "currentTabLoaded")) {
        nav.val(getLocalStorageItem(dashboard["type"] + "currentTabLoaded"));
    }
    nav.trigger("change");
}

function loadEchartsDashboardTab(element) {
    var dashboard = echartsDashboardData;
    var contentId = dashboard["container"] + "_content";
    $('.' + contentId).hide();
    if (element.val() !== "") {
        setLocalStorageItem(dashboard["type"] + "currentTabLoaded", element.val());
        var container = $("#" + element.val());
        if (!(".front-viewport-body").length) {
            showOverlay();
        }
        setTimeout(function () {
            container.fadeIn();
            if (container.html() === "") {
                var tabKey = element.val().split("___")[1];
                var tab = dashboard["tabs"][tabKey];
                renderEchartsDashboardStructure(container, tab);
            }
            if (!(".front-viewport-body").length) {
                hideOverlay();
            }
            resizeEcharts();
        }, 0);
    }
    return true;
}

function renderEchartsDashboardStructure(parentContainer, structure) {
    if (!structure["components"]) {
        return false;
    }
    $.each(structure["components"], function (k, v) {
        var containerId = parentContainer.attr("id") + "_" + k;
        if (!$("#" + containerId).length) {
            parentContainer.append('<div class="' + (v["class"] ? v["class"] : "") + '"><div id="' + containerId + '" class="row dashboard-component-container" style="' + (v["style"] ? v["style"] : "") + ';"></div></div>');
        }
        if (v["type"] === "chart") {
            window[v["function"]](containerId, v["data"]);
        } else if (v["type"] === "html") {
            $("#" + containerId).html(v["data"]);
        } else if (v["type"] === "container") {
            renderEchartsDashboardStructure($("#" + containerId), v);
        }
    });
    return true;
}

function pieTooltipFormater(params, ticket, callback) {
    var value = params.value;
    if (!isNaN(parseFloat(value)) || params.data.numberFormatValue === 1) {
        value = appNumberFormat(value);
    }
    if (params.data.currencyValue === 1) {
        value = value + ' ' + AppConfig.currencySymbol;
    }
    var res = "<span style='background-color:" + params.color + ";'>&nbsp;&nbsp;</span> " + params.name;
    res += '<br>' + value;
    if (typeof params.percent !== "undefined")
        res += ' (' + appNumberFormat(params.percent) + '%)';
    if (typeof params.data.amount !== "undefined")
        res += '<br>' + appNumberFormat(params.data.amount) + ' ' + AppConfig.currencySymbol;
    if (typeof params.data.extraTooltipInfo !== "undefined") {
        $.each(params.data.extraTooltipInfo, function (k, v) {
            res += '<br>' + v;
        });
    }
    return res;
}

function pieLegendFormatter(name) {
    return name.substring(0, 15);
}

function sunburstTooltipFormater(params, ticket, callback) {
    var value = params.value;
    if (!isNaN(parseFloat(value)) || params.data.numberFormatValue === 1) {
        value = appNumberFormat(value);
    }
    if (params.data.currencyValue === 1) {
        value = value + ' ' + AppConfig.currencySymbol;
    }
    var res = "<span style='background-color:" + params.color + ";'>&nbsp;&nbsp;</span> " + params["treePathInfo"][1]["name"];
    if (params["treePathInfo"][2]) {
        res += "<br>" + params["treePathInfo"][2]["name"];
    }
    var percent = 100 * (params["value"] / params["treePathInfo"][0]["value"]);
    res += '<br>' + value;
    if (!isNaN(percent))
        res += ' (' + appNumberFormat(percent) + '%)';
    if (typeof params.data.amount !== "undefined")
        res += '<br>' + appNumberFormat(params.data.amount) + ' ' + AppConfig.currencySymbol;
    if (typeof params.data.extraTooltipInfo !== "undefined") {
        $.each(params.data.extraTooltipInfo, function (k, v) {
            res += '<br>' + v;
        });
    }
    return res;
}

/*
 * Este tipo de gráfica no permite leyenda así que vamos a ajustar la etiqueta para que se vea pero no sea muy grande
 */
function sunburstLabelFormater(params) {
    var percent = parseFloat(100 * (params["value"] / params["treePathInfo"][0]["value"]));
    if (percent < 10) {
        return ' ';
    }
    return params["name"].substring(0, 10);
}

function lineTooltipFormater(params, ticket, callback, hideTotal) {
    var res = "";
    var total = 0;
    var i = 0;
    var itemsPerLine = 2;

    $.each(params, function (k, v) {
        if (res === "") {
            res += '<table><tr><td colspan="' + (itemsPerLine * 2) + '" align="center"><b>' + v.name + '</b></td></tr>';
            res += '<tr style="height:14px;">';
        }
        if (v.value !== 0) {
            if (i !== 0 && i % itemsPerLine === 0) {
                res += '</tr><tr style="height:14px;">';
            }
            res += '<td style="font-size:12px;"><span style="background-color:' + v.color + ';">&nbsp;&nbsp;</span> ' + v.seriesName + '</td>';
            res += '<td align="right" style="font-size:12px;">' + numberFormat(v.value) + '</td>';
            i++;
            total += v.value;
        }
    });

    res += "</tr>";
    if (hideTotal !== true) {
        res += '<tr><td colspan="' + (itemsPerLine * 2) + '" align="center"><b>Total</b>: ' + numberFormat(total.toFixed(2)) + '</td></tr>';
    }
    res += "</table>";
    return res;
}

function lineTooltipFormaterHideTotal(params, ticket, callback) {
    return lineTooltipFormater(params, ticket, callback, true);
}

function lineLegendFormatter(name) {
    return name.substring(0, 15);
}

function echartsMapTooltipFormater(params, ticket, callback) {
    return params.seriesName + '<br/><b>' + params.name + '</b>: ' + appNumberFormat(params.value);
}

function barTooltipFormater(params, ticket, callback, hideTotal) {
    if (typeof hideTotal === "undefined") {
        hideTotal = true;
    }
    return lineTooltipFormater(params, ticket, callback, hideTotal);
}

function barTooltipFormaterPercentage(params, ticket, callback) {
    var res = "";
    var total = 0;
    var i = 0;
    var values = [];
    $.each(params, function (k, v) {
        if (res === "") {
            res += "<div style='width: 100%;text-align:center;'><b>" + v.name + "</b></div>";
        }
        if (v.value !== 0) {
            values[i] = v.value;
            res += "<div style='width: 100%;'>" + v.seriesName + ": " + numberFormat(v.value) + "</div>";
            i = i + 1;
        }
        total += v.value;
    });
    if (i === 2) {
        res += "<div style='width: 100%;'>" + numberFormat(100 * (values[1] / values[0])) + " %</div>";
    }
    return res;
}

function barTooltipFormaterTotal(params, ticket, callback) {
    return barTooltipFormater(params, ticket, callback, false);
}

function funnelTooltipFormater(params, ticket, callback) {
    var res = "";
    res += "<div style='width: 100%;'>" + params.name + ": " + numberFormat(params.value) + "</div>";
    return res;
}

function axisLabelFormatter(value) {
    if (isNaN(parseFloat(value))) {
        return value;
    } else if (Number.isInteger(parseFloat(value))) {
        return appNumberFormat(value, 0);
    }
    return appNumberFormat(value);
}

function gaugeTooltipFormater(value) {
    if (value.value) {
        value = value.value;
    }
    var res = numberFormat(value) + "%";
    return res;
}

function generateTitleObject(data) {
    var item = {
        text: data.title, textStyle: (data.titleTextStyle ? data.titleTextStyle : {fontWeight: 'normal'}),
        x: (data.titleItemX ? data.titleItemX : 'center'), y: (data.titleItemY ? data.titleItemY : 'auto')
    };
    if (data.subtext) {
        item["subtext"] = data.subtext;
        item["subtextStyle"] = (data.subtextTextStyle ? data.subtextTextStyle : {fontWeight: 'normal'});
    }
    return item;
}

function generateLegendItemObject(name, icon) {
    var item = {
        name: name
    };
    if (typeof icon !== "undefined") {
        item.icon = icon;
    }
    return item;
}

function generateLegendObject(legendData, formatter) {
    var item = {
        orient: 'horizontal',
        bottom: 0,
        itemGap: 4,
        itemWidth: 8,
        data: legendData,
        textStyle: {
            fontSize: 12,
            lineHeight: 12
        }
    };
    if (typeof formatter !== "undefined") {
        item.formatter = formatter;
    }
    if (AppConfig["templateThemeVersion"] >= 2 || legendData.length && legendData.length > 10) {
        item.type = 'scroll';
    }
    return item;
}

function generateToolboxObject(data) {
    var item = {
        show: true,
        feature: {
            dataView: {show: (data.enableDataView ? true : false), readOnly: true, title: ' '},
            saveAsImage: {show: (data.disableExport ? false : true), title: ' '},
            restore: {show: (data.enableRestore ? true : false), title: ' '},
            magicType: (data.magicType ? data.magicType : null)
        }
    };
    return item;
}

function calcEchartsTooltipPosition(point, params, dom, rect, size) {
    var obj = {top: (point[1] + 20)};
    obj[['left', 'right'][+(point[0] < size.viewSize[0] / 2)]] = 5;
    return obj;
}

function getEchartsDataIndexData(echartsContainer, seriesIndex, dataIndex, childrenNodeKey) {
    if (typeof childrenNodeKey === "undefined") {
        childrenNodeKey = "children";
    }
    var lastSearchIndex = 0;
    var dataIndexData = false;
    var echarts = EchartsCRMV3[echartsContainer];
    var serie = echarts.getOption("data")["series"][seriesIndex];
    var serieData = serie["data"];
    $.each(serieData, echartsDataIndexDataWalker);

    function echartsDataIndexDataWalker(key, value) {
        if (dataIndexData !== false) {
            return false;
        }
        lastSearchIndex++;
        if (lastSearchIndex === dataIndex) {
            dataIndexData = value;
            return true;
        }
        if (typeof value === "object" && value[childrenNodeKey]) {
            $.each(value[childrenNodeKey], echartsDataIndexDataWalker);
        }
    }
    return dataIndexData;
}

function registerEchartsMap(countryCode, callback) {
    var mapName = 'echartsMap' + countryCode;
    if (EchartsMapsDataCRMV3[mapName]) {
        //Ya tenemos el papa registrado
        var mapData = EchartsMapsDataCRMV3[mapName];
        if (typeof callback !== "undefined") {
            callback({"mapName": mapName, "mapData": mapData});
        }
    } else {
        //No tenemos el mapa registrado, hacemos petición ajax
        var params = "class=Country&method=getCountryGeoJson&country=" + countryCode + "&outputFormat=JSON";
        if (!EchartsMapsDataCRMV3Registering[params] || EchartsMapsDataCRMV3Registering[params] !== 1) {
            //Es la primera vez que pedidos el mapa en esta ejecución
            //Ponemos bandera para indicar que estamos registrando el mapa
            EchartsMapsDataCRMV3Registering[params] = 1;
            callClassMethod(
                    params,
                    true,
                    function (data) {
                        var mapData = JSON.parse(data);
                        echarts.registerMap(mapName, mapData, mapData["specialAreas"]);
                        EchartsMapsDataCRMV3[mapName] = mapData;
                        EchartsMapsDataCRMV3Registering[params] = 0;
                        callback({"mapName": mapName, "mapData": mapData});
                    }
            );
        } else {
            //Si estamos pidiendo el mapa mas veces de golpe esperamos a que la primera petición lo registre para ahorrarnos llamadas ya que los mapas pesan
            var registerEchartsMapTimeout = setTimeout(function () {
                if (EchartsMapsDataCRMV3Registering[params] === 0) {
                    //Si la primera solicitud ya ha terminado cancelamos el timeout y volvemos a pedir el mapa ya que ahora ya lo tendremos
                    clearTimeout(registerEchartsMapTimeout);
                    registerEchartsMap(countryCode, callback);
                }
            }, 1000);
        }
    }
    return true;
}
